@@ -11,17 +11,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import json
-import os
-from datetime import datetime
+
 from pathlib import Path
 from typing import Optional
-
-import click
+from click import command, option, argument, Choice
 
 from lean.click import LeanCommand, PathParameter
-from lean.constants import DEFAULT_ENGINE_IMAGE
-from lean.container import container
+from lean.constants import DEFAULT_ENGINE_IMAGE, LEAN_ROOT_PATH
+from lean.container import container, Logger
 from lean.models.api import QCMinimalOrganization
 from lean.models.utils import DebuggingMethod
 from lean.models.logger import Option
@@ -36,12 +33,15 @@ from lean.models.data_providers import QuantConnectDataProvider, all_data_provid
 #
 # These methods checks if the project has outdated configurations, and if so, update them to keep it working.
 
-def _migrate_python_pycharm(project_dir: Path) -> None:
+def _migrate_python_pycharm(logger: Logger, project_dir: Path) -> None:
+    from os import path
+    from click import Abort
+
     workspace_xml_path = project_dir / ".idea" / "workspace.xml"
     if not workspace_xml_path.is_file():
         return
 
-    xml_manager = container.xml_manager()
+    xml_manager = container.xml_manager
     current_content = xml_manager.parse(workspace_xml_path.read_text(encoding="utf-8"))
 
     config = current_content.find('.//configuration[@name="Debug with Lean CLI"]')
@@ -55,14 +55,15 @@ def _migrate_python_pycharm(project_dir: Path) -> None:
     made_changes = False
     has_library_mapping = False
 
-    library_dir = container.lean_config_manager().get_cli_root_directory() / "Library"
+    library_dir = container.lean_config_manager.get_cli_root_directory() / "Library"
+
     if library_dir.is_dir():
-        library_dir = f"$PROJECT_DIR$/{os.path.relpath(library_dir, project_dir)}".replace("\\", "/")
+        library_dir = f"$PROJECT_DIR$/{path.relpath(library_dir, project_dir)}".replace("\\", "/")
     else:
         library_dir = None
 
     for mapping in path_mappings.findall(".//mapping"):
-        if mapping.get("local-root") == "$PROJECT_DIR$" and mapping.get("remote-root") == "/Lean/Launcher/bin/Debug":
+        if mapping.get("local-root") == "$PROJECT_DIR$" and mapping.get("remote-root") == LEAN_ROOT_PATH:
             mapping.set("remote-root", "/LeanCLI")
             made_changes = True
 
@@ -81,19 +82,20 @@ def _migrate_python_pycharm(project_dir: Path) -> None:
     if made_changes:
         workspace_xml_path.write_text(xml_manager.to_string(current_content), encoding="utf-8")
 
-        logger = container.logger()
+        logger = container.logger
         logger.warn("Your run configuration has been updated to work with the latest version of LEAN")
         logger.warn("Please restart the debugger in PyCharm and run this command again")
 
-        raise click.Abort()
+        raise Abort()
 
 
 def _migrate_python_vscode(project_dir: Path) -> None:
+    from json import dumps, loads
     launch_json_path = project_dir / ".vscode" / "launch.json"
     if not launch_json_path.is_file():
         return
 
-    current_content = json.loads(launch_json_path.read_text(encoding="utf-8"))
+    current_content = loads(launch_json_path.read_text(encoding="utf-8"))
     if "configurations" not in current_content or not isinstance(current_content["configurations"], list):
         return
 
@@ -104,12 +106,12 @@ def _migrate_python_vscode(project_dir: Path) -> None:
     made_changes = False
     has_library_mapping = False
 
-    library_dir = container.lean_config_manager().get_cli_root_directory() / "Library"
+    library_dir = container.lean_config_manager.get_cli_root_directory() / "Library"
     if not library_dir.is_dir():
         library_dir = None
 
     for mapping in config["pathMappings"]:
-        if mapping["localRoot"] == "${workspaceFolder}" and mapping["remoteRoot"] == "/Lean/Launcher/bin/Debug":
+        if mapping["localRoot"] == "${workspaceFolder}" and mapping["remoteRoot"] == LEAN_ROOT_PATH:
             mapping["remoteRoot"] = "/LeanCLI"
             made_changes = True
 
@@ -124,12 +126,14 @@ def _migrate_python_vscode(project_dir: Path) -> None:
         made_changes = True
 
     if made_changes:
-        launch_json_path.write_text(json.dumps(current_content, indent=4), encoding="utf-8")
+        launch_json_path.write_text(dumps(current_content, indent=4), encoding="utf-8")
 
 
-def _migrate_csharp_rider(project_dir: Path) -> None:
+def _migrate_csharp_rider(logger: Logger, project_dir: Path) -> None:
+    from click import Abort
+
     made_changes = False
-    xml_manager = container.xml_manager()
+    xml_manager = container.xml_manager
 
     for dir_name in [f".idea.{project_dir.stem}", f".idea.{project_dir.stem}.dir"]:
         workspace_xml_path = project_dir / ".idea" / dir_name / ".idea" / "workspace.xml"
@@ -152,23 +156,23 @@ def _migrate_csharp_rider(project_dir: Path) -> None:
         made_changes = True
 
     if made_changes:
-        container.project_manager().generate_rider_config()
+        container.project_manager.generate_rider_config()
 
-        logger = container.logger()
         logger.warn("Your run configuration has been updated to work with the .NET 5 version of LEAN")
         logger.warn("Please restart Rider and start debugging again")
         logger.warn(
-            "See https://www.lean.io/docs/lean-cli/backtesting/debugging#05-C-and-Rider for the updated instructions")
+            "See https://www.lean.io/docs/v2/lean-cli/backtesting/debugging#05-C-and-Rider for the updated instructions")
 
-        raise click.Abort()
+        raise Abort()
 
 
 def _migrate_csharp_vscode(project_dir: Path) -> None:
+    from json import dumps, loads
     launch_json_path = project_dir / ".vscode" / "launch.json"
     if not launch_json_path.is_file():
         return
 
-    current_content = json.loads(launch_json_path.read_text(encoding="utf-8"))
+    current_content = loads(launch_json_path.read_text(encoding="utf-8"))
     if "configurations" not in current_content or not isinstance(current_content["configurations"], list):
         return
 
@@ -197,7 +201,7 @@ def _migrate_csharp_vscode(project_dir: Path) -> None:
         "moduleLoad": False
     }
 
-    launch_json_path.write_text(json.dumps(current_content, indent=4), encoding="utf-8")
+    launch_json_path.write_text(dumps(current_content, indent=4), encoding="utf-8")
 
 
 def _migrate_csharp_csproj(project_dir: Path) -> None:
@@ -205,7 +209,7 @@ def _migrate_csharp_csproj(project_dir: Path) -> None:
     if csproj_path is None:
         return
 
-    xml_manager = container.xml_manager()
+    xml_manager = container.xml_manager
 
     current_content = xml_manager.parse(csproj_path.read_text(encoding="utf-8"))
     if current_content.find(".//PropertyGroup/DefaultItemExcludes") is not None:
@@ -228,48 +232,54 @@ def _select_organization() -> QCMinimalOrganization:
 
     :return: the selected organization
     """
-    api_client = container.api_client()
+    api_client = container.api_client
 
     organizations = api_client.organizations.get_all()
     options = [Option(id=organization, label=organization.name) for organization in organizations]
 
-    logger = container.logger()
+    logger = container.logger
     return logger.prompt_list("Select the organization to purchase and download data with", options)
 
 
-@click.command(cls=LeanCommand, requires_lean_config=True, requires_docker=True)
-@click.argument("project", type=PathParameter(exists=True, file_okay=True, dir_okay=True))
-@click.option("--output",
+@command(cls=LeanCommand, requires_lean_config=True, requires_docker=True)
+@argument("project", type=PathParameter(exists=True, file_okay=True, dir_okay=True))
+@option("--output",
               type=PathParameter(exists=False, file_okay=False, dir_okay=True),
               help="Directory to store results in (defaults to PROJECT/backtests/TIMESTAMP)")
-@click.option("--detach", "-d",
+@option("--detach", "-d",
               is_flag=True,
               default=False,
               help="Run the backtest in a detached Docker container and return immediately")
-@click.option("--debug",
-              type=click.Choice(["pycharm", "ptvsd", "vsdbg", "rider"], case_sensitive=False),
+@option("--debug",
+              type=Choice(["pycharm", "ptvsd", "vsdbg", "rider"], case_sensitive=False),
               help="Enable a certain debugging method (see --help for more information)")
-@click.option("--data-provider",
-              type=click.Choice([dp.get_name() for dp in all_data_providers], case_sensitive=False),
+@option("--data-provider",
+              type=Choice([dp.get_name() for dp in all_data_providers], case_sensitive=False),
               help="Update the Lean configuration file to retrieve data from the given provider")
-@click.option("--download-data",
+@option("--download-data",
               is_flag=True,
               default=False,
               help="Update the Lean configuration file to download data from the QuantConnect API, alias for --data-provider QuantConnect")
-@click.option("--data-purchase-limit",
+@option("--data-purchase-limit",
               type=int,
               help="The maximum amount of QCC to spend on downloading data during the backtest when using QuantConnect as data provider")
-@click.option("--release",
+@option("--release",
               is_flag=True,
               default=False,
               help="Compile C# projects in release configuration instead of debug")
-@click.option("--image",
+@option("--image",
               type=str,
               help=f"The LEAN engine image to use (defaults to {DEFAULT_ENGINE_IMAGE})")
-@click.option("--update",
+@option("--python-venv",
+              type=str,
+              help=f"The path of the python virtual environment to be used")
+@option("--update",
               is_flag=True,
               default=False,
               help="Pull the LEAN engine image before running the backtest")
+@option("--backtest-name",
+              type=str,
+              help="Backtest name")
 def backtest(project: Path,
              output: Optional[Path],
              detach: bool,
@@ -279,7 +289,9 @@ def backtest(project: Path,
              data_purchase_limit: Optional[int],
              release: bool,
              image: Optional[str],
-             update: bool) -> None:
+             python_venv: Optional[str],
+             update: bool,
+             backtest_name: str) -> None:
     """Backtest a project locally using Docker.
 
     \b
@@ -288,23 +300,24 @@ def backtest(project: Path,
 
     \b
     Go to the following url to learn how to debug backtests locally using the Lean CLI:
-    https://www.lean.io/docs/lean-cli/backtesting/debugging
+    https://www.lean.io/docs/v2/lean-cli/backtesting/debugging
 
     By default the official LEAN engine image is used.
     You can override this using the --image option.
     Alternatively you can set the default engine image for all commands using `lean config set engine-image <image>`.
     """
-    project_manager = container.project_manager()
+    from datetime import datetime
+    logger = container.logger
+    project_manager = container.project_manager
     algorithm_file = project_manager.find_algorithm_file(Path(project))
-    lean_config_manager = container.lean_config_manager()
-
+    lean_config_manager = container.lean_config_manager
     if output is None:
         output = algorithm_file.parent / "backtests" / datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
 
     debugging_method = None
     if debug == "pycharm":
         debugging_method = DebuggingMethod.PyCharm
-        _migrate_python_pycharm(algorithm_file.parent)
+        _migrate_python_pycharm(logger, algorithm_file.parent)
     elif debug == "ptvsd":
         debugging_method = DebuggingMethod.PTVSD
         _migrate_python_vscode(algorithm_file.parent)
@@ -313,7 +326,7 @@ def backtest(project: Path,
         _migrate_csharp_vscode(algorithm_file.parent)
     elif debug == "rider":
         debugging_method = DebuggingMethod.Rider
-        _migrate_csharp_rider(algorithm_file.parent)
+        _migrate_csharp_rider(logger, algorithm_file.parent)
 
     if debugging_method is not None and detach:
         raise RuntimeError("Running a debugging session in a detached container is not supported")
@@ -328,25 +341,35 @@ def backtest(project: Path,
 
     if data_provider is not None:
         data_provider = next(dp for dp in all_data_providers if dp.get_name() == data_provider)
-        data_provider.build(lean_config, container.logger()).configure(lean_config, "backtesting")
+        data_provider.build(lean_config, logger).configure(lean_config, "backtesting")
 
     lean_config_manager.configure_data_purchase_limit(lean_config, data_purchase_limit)
 
-    cli_config_manager = container.cli_config_manager()
-    project_config_manager = container.project_config_manager()
+    cli_config_manager = container.cli_config_manager
+    project_config_manager = container.project_config_manager
 
     project_config = project_config_manager.get_project_config(algorithm_file.parent)
     engine_image = cli_config_manager.get_engine_image(image or project_config.get("engine-image", None))
 
-    container.update_manager().pull_docker_image_if_necessary(engine_image, update)
+    if str(engine_image) != DEFAULT_ENGINE_IMAGE:
+        logger.warn(f'A custom engine image: "{engine_image}" is being used!')
+
+    container.update_manager.pull_docker_image_if_necessary(engine_image, update)
 
     if not output.exists():
         output.mkdir(parents=True)
 
-    output_config_manager = container.output_config_manager()
+    output_config_manager = container.output_config_manager
     lean_config["algorithm-id"] = str(output_config_manager.get_backtest_id(output))
 
-    lean_runner = container.lean_runner()
+    # Set backtest name
+    if backtest_name is not None and backtest_name != "":
+        lean_config["backtest-name"] = backtest_name
+
+    if python_venv is not None and python_venv != "":
+        lean_config["python-venv"] = f'{"/" if python_venv[0] != "/" else ""}{python_venv}'
+
+    lean_runner = container.lean_runner
     lean_runner.run_lean(lean_config,
                          "backtesting",
                          algorithm_file,
