@@ -11,16 +11,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import re
 from pathlib import Path
 from typing import Optional
 
-import click
+from click import option, command, argument
 
 from lean.click import LeanCommand, PathParameter
 from lean.container import container
 from lean.models.docker import DockerImage
-
+from lean.constants import CUSTOM_FOUNDATION, CUSTOM_RESEARCH, CUSTOM_ENGINE
 
 def _normalize_newlines(text: str) -> str:
     """Normalizes the newlines in a string to use \n (instead of \r or \r\n).
@@ -41,7 +40,7 @@ def _is_foundation_dockerfile_same_as_cloud(dockerfile: Path) -> bool:
 
     try:
         cloud_url = f"https://raw.githubusercontent.com/QuantConnect/Lean/master/{dockerfile.name}"
-        cloud_dockerfile = container.http_client().get(cloud_url)
+        cloud_dockerfile = container.http_client.get(cloud_url)
         cloud_dockerfile = cloud_dockerfile.text.strip()
     except:
         # We build a new image if for whatever reason we can't check the Dockerfile used for the official image
@@ -57,12 +56,12 @@ def _compile_csharp(root: Path, csharp_dir: Path, docker_image: DockerImage) ->
     :param csharp_dir: the directory containing the C# code
     :param docker_image: the Docker image to compile in
     """
-    logger = container.logger()
+    logger = container.logger
     logger.info(f"Compiling the C# code in '{csharp_dir}'")
 
     build_path = Path("/LeanCLI") / csharp_dir.relative_to(root)
 
-    docker_manager = container.docker_manager()
+    docker_manager = container.docker_manager
     docker_manager.create_volume("lean_cli_nuget")
     success = docker_manager.run_image(docker_image,
                                        entrypoint=["dotnet", "build", str(build_path)],
@@ -93,7 +92,9 @@ def _build_image(root: Path, dockerfile: Path, base_image: Optional[DockerImage]
     :param base_image: the base image to use, or None if the default should be used
     :param target_image: the name of the new image
     """
-    logger = container.logger()
+    from re import MULTILINE, sub
+
+    logger = container.logger
     if base_image is not None:
         logger.info(f"Building '{target_image}' from '{dockerfile}' using '{base_image}' as base image")
     else:
@@ -105,26 +106,26 @@ def _build_image(root: Path, dockerfile: Path, base_image: Optional[DockerImage]
     current_content = dockerfile.read_text(encoding="utf-8")
 
     if base_image is not None:
-        new_content = re.sub(r"^FROM.*$", f"FROM {base_image}", current_content, flags=re.MULTILINE)
+        new_content = sub(r"^FROM.*$", f"FROM {base_image}", current_content, flags=MULTILINE)
         dockerfile.write_text(new_content, encoding="utf-8")
 
     try:
-        docker_manager = container.docker_manager()
+        docker_manager = container.docker_manager
         docker_manager.build_image(root, dockerfile, target_image)
     finally:
         if base_image is not None:
             dockerfile.write_text(current_content, encoding="utf-8")
 
 
-@click.command(cls=LeanCommand, requires_docker=True)
-@click.argument("root", type=PathParameter(exists=True, file_okay=False, dir_okay=True), default=lambda: Path.cwd())
-@click.option("--tag", type=str, default="latest", help="The tag to apply to custom images (defaults to latest)")
+@command(cls=LeanCommand, requires_docker=True)
+@argument("root", type=PathParameter(exists=True, file_okay=False, dir_okay=True), default=lambda: Path.cwd())
+@option("--tag", type=str, default="latest", help="The tag to apply to custom images (defaults to latest)")
 def build(root: Path, tag: str) -> None:
-    """Build Docker images of your own version of LEAN and the Alpha Streams SDK.
+    """Build Docker images of your own version of LEAN.
 
     \b
-    ROOT must point to a directory containing the LEAN repository and the Alpha Streams SDK repository:
-    https://github.com/QuantConnect/Lean & https://github.com/QuantConnect/AlphaStreams
+    ROOT must point to a directory containing the LEAN repository:
+    https://github.com/QuantConnect/Lean
 
     When ROOT is not given, the current directory is used as root directory.
 
@@ -132,11 +133,10 @@ def build(root: Path, tag: str) -> None:
     This command performs the following actions:
     1. The lean-cli/foundation:latest image is built from Lean/DockerfileLeanFoundation(ARM).
     2. LEAN is compiled in a Docker container using the lean-cli/foundation:latest image.
-    3. The Alpha Streams SDK is compiled in a Docker container using the lean-cli/foundation:latest image.
-    4. The lean-cli/engine:latest image is built from Lean/Dockerfile using lean-cli/foundation:latest as base image.
-    5. The lean-cli/research:latest image is built from Lean/DockerfileJupyter using lean-cli/engine:latest as base image.
-    6. The default engine image is set to lean-cli/engine:latest.
-    7. The default research image is set to lean-cli/research:latest.
+    3. The lean-cli/engine:latest image is built from Lean/Dockerfile using lean-cli/foundation:latest as base image.
+    4. The lean-cli/research:latest image is built from Lean/DockerfileJupyter using lean-cli/engine:latest as base image.
+    5. The default engine image is set to lean-cli/engine:latest.
+    6. The default research image is set to lean-cli/research:latest.
 
     When the foundation Dockerfile is the same as the official foundation Dockerfile,
     quantconnect/lean:foundation is used instead of building a custom foundation image.
@@ -145,35 +145,30 @@ def build(root: Path, tag: str) -> None:
     if not lean_dir.is_dir():
         raise RuntimeError(f"Please clone https://github.com/QuantConnect/Lean to '{lean_dir}'")
 
-    alpha_streams_dir = root / "AlphaStreams"
-    if not lean_dir.is_dir():
-        raise RuntimeError(f"Please clone https://github.com/QuantConnect/AlphaStreams to '{alpha_streams_dir}'")
-
     (root / "DataLibraries").mkdir(exist_ok=True)
 
-    if container.platform_manager().is_host_arm():
+    if container.platform_manager.is_host_arm():
         foundation_dockerfile = lean_dir / "DockerfileLeanFoundationARM"
     else:
         foundation_dockerfile = lean_dir / "DockerfileLeanFoundation"
 
     if _is_foundation_dockerfile_same_as_cloud(foundation_dockerfile):
         foundation_image = DockerImage(name="quantconnect/lean", tag="foundation")
-        container.docker_manager().pull_image(foundation_image)
+        container.docker_manager.pull_image(foundation_image)
     else:
-        foundation_image = DockerImage(name="lean-cli/foundation", tag=tag)
+        foundation_image = DockerImage(name=CUSTOM_FOUNDATION, tag=tag)
         _build_image(root, foundation_dockerfile, None, foundation_image)
 
     _compile_csharp(root, lean_dir, foundation_image)
-    _compile_csharp(root, alpha_streams_dir, foundation_image)
 
-    custom_engine_image = DockerImage(name="lean-cli/engine", tag=tag)
+    custom_engine_image = DockerImage(name=CUSTOM_ENGINE, tag=tag)
     _build_image(root, lean_dir / "Dockerfile", foundation_image, custom_engine_image)
 
-    custom_research_image = DockerImage(name="lean-cli/research", tag=tag)
+    custom_research_image = DockerImage(name=CUSTOM_RESEARCH, tag=tag)
     _build_image(root, lean_dir / "DockerfileJupyter", custom_engine_image, custom_research_image)
 
-    logger = container.logger()
-    cli_config_manager = container.cli_config_manager()
+    logger = container.logger
+    cli_config_manager = container.cli_config_manager
 
     logger.info(f"Setting default engine image to '{custom_engine_image}'")
     cli_config_manager.engine_image.set_value(str(custom_engine_image))
