@@ -14,56 +14,62 @@
 from datetime import datetime
 from typing import Optional
 
-import click
+from click import command, option, Choice, IntRange
 
 from lean.click import DateParameter, LeanCommand
 from lean.constants import DEFAULT_ENGINE_IMAGE
 from lean.container import container
 
 
-@click.command(cls=LeanCommand, requires_lean_config=True, requires_docker=True)
-@click.option("--start",
+@command(cls=LeanCommand, requires_lean_config=True, requires_docker=True)
+@option("--start",
               type=DateParameter(),
               required=True,
               help="Start date for the data to generate in yyyyMMdd format")
-@click.option("--end",
+@option("--end",
               type=DateParameter(),
               default=datetime.today().strftime("%Y%m%d"),
               help="End date for the data to generate in yyyyMMdd format (defaults to today)")
-@click.option("--symbol-count",
-              type=click.IntRange(min=0),
+@option("--symbol-count",
+              type=IntRange(min=0),
               required=True,
               help="The number of symbols to generate data for")
-@click.option("--security-type",
-              type=click.Choice(["Equity", "Forex", "Cfd", "Future", "Crypto", "Option"], case_sensitive=False),
+@option("--tickers",
+              type=str,
+              required=False,
+              default="",
+              help="Comma separated list of tickers to use for generated data")
+@option("--security-type",
+              type=Choice(["Equity", "Forex", "Cfd", "Future", "Crypto", "Option"], case_sensitive=False),
               default="Equity",
               help="The security type to generate data for (defaults to Equity)")
-@click.option("--resolution",
-              type=click.Choice(["Tick", "Second", "Minute", "Hour", "Daily"], case_sensitive=False),
+@option("--resolution",
+              type=Choice(["Tick", "Second", "Minute", "Hour", "Daily"], case_sensitive=False),
               default="Minute",
               help="The resolution of the generated data (defaults to Minute)")
-@click.option("--data-density",
-              type=click.Choice(["Dense", "Sparse", "VerySparse"], case_sensitive=False),
+@option("--data-density",
+              type=Choice(["Dense", "Sparse", "VerySparse"], case_sensitive=False),
               default="Dense",
               help="The density of the generated data (defaults to Dense)")
-@click.option("--include-coarse",
+@option("--include-coarse",
               type=bool,
               default=True,
               help="Whether coarse universe data should be generated for Equity data (defaults to True)")
-@click.option("--market",
+@option("--market",
               type=str,
               default="",
               help="The market to generate data for (defaults to standard market for the security type)")
-@click.option("--image",
+@option("--image",
               type=str,
               help=f"The LEAN engine image to use (defaults to {DEFAULT_ENGINE_IMAGE})")
-@click.option("--update",
+@option("--update",
               is_flag=True,
               default=False,
               help="Pull the LEAN engine image before running the generator")
 def generate(start: datetime,
              end: datetime,
              symbol_count: int,
+             tickers: str,
              security_type: str,
              resolution: str,
              data_density: str,
@@ -103,21 +109,27 @@ def generate(start: datetime,
     You can override this using the --image option.
     Alternatively you can set the default engine image for all commands using `lean config set engine-image <image>`.
     """
-    lean_config_manager = container.lean_config_manager()
+    lean_config_manager = container.lean_config_manager
     data_dir = lean_config_manager.get_data_directory()
 
+    entrypoint = ["dotnet", "QuantConnect.ToolBox.dll",
+                  "--destination-dir", "/Lean/Data",
+                  "--app", "randomdatagenerator",
+                  "--start", start.strftime("%Y%m%d"),
+                  "--end", end.strftime("%Y%m%d"),
+                  "--symbol-count", str(symbol_count),
+                  "--security-type", security_type,
+                  "--resolution", resolution,
+                  "--data-density", data_density,
+                  "--include-coarse", str(include_coarse).lower(),
+                  "--market", market.lower()]
+
+    # Toolbox uses '--opt=val' as single argument
+    if tickers:
+        entrypoint.append("--tickers=" + tickers)
+
     run_options = {
-        "entrypoint": ["dotnet", "QuantConnect.ToolBox.dll",
-                       "--destination-dir", "/Lean/Data",
-                       "--app", "randomdatagenerator",
-                       "--start", start.strftime("%Y%m%d"),
-                       "--end", end.strftime("%Y%m%d"),
-                       "--symbol-count", str(symbol_count),
-                       "--security-type", security_type,
-                       "--resolution", resolution,
-                       "--data-density", data_density,
-                       "--include-coarse", str(include_coarse).lower(),
-                       "--market", market.lower()],
+        "entrypoint": entrypoint,
         "volumes": {
             str(data_dir): {
                 "bind": "/Lean/Data",
@@ -126,11 +138,11 @@ def generate(start: datetime,
         }
     }
 
-    engine_image = container.cli_config_manager().get_engine_image(image)
+    engine_image = container.cli_config_manager.get_engine_image(image)
 
-    container.update_manager().pull_docker_image_if_necessary(engine_image, update)
+    container.update_manager.pull_docker_image_if_necessary(engine_image, update)
 
-    success = container.docker_manager().run_image(engine_image, **run_options)
+    success = container.docker_manager.run_image(engine_image, **run_options)
     if not success:
         raise RuntimeError(
             "Something went wrong while running the random data generator, see the logs above for more information")
