@@ -11,18 +11,10 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import json
-import platform
-import shutil
-import subprocess
-from distutils.version import StrictVersion
 from pathlib import Path
 from typing import Any, Dict, Optional, Tuple
-
-import click
-from dateutil.parser import isoparse
-from lxml import etree
-from pkg_resources import Requirement
+from lean.constants import LEAN_STRICT_PYTHON_VERSION
+from click import command, argument, option
 
 from lean.click import LeanCommand, PathParameter
 from lean.container import container
@@ -35,14 +27,15 @@ def _get_nuget_package(name: str) -> Tuple[str, str]:
     :param name: the name of the package, case in-sensitive
     :return: a tuple containing the proper name and latest version of the package, excluding pre-release versions
     """
-    http_client = container.http_client()
+    from json import loads
+    http_client = container.http_client
     generic_error = RuntimeError(f"The NuGet API is not responding")
 
     service_index_response = http_client.get("https://api.nuget.org/v3/index.json", raise_for_status=False)
     if not service_index_response.ok:
         raise generic_error
 
-    service_index = json.loads(service_index_response.text)
+    service_index = loads(service_index_response.text)
     query_url = next((x["@id"] for x in service_index["resources"] if x["@type"] == "SearchQueryService"), None)
     if query_url is None:
         raise generic_error
@@ -51,7 +44,7 @@ def _get_nuget_package(name: str) -> Tuple[str, str]:
     if not query_response.ok:
         raise generic_error
 
-    query_results = json.loads(query_response.text)
+    query_results = loads(query_response.text)
     package_data = next((p for p in query_results["data"] if p["id"].lower() == name.lower()), None)
 
     if package_data is None:
@@ -67,7 +60,8 @@ def _add_csharp_package_to_csproj(csproj_file: Path, name: str, version: str) ->
     :param name: the name of the package
     :param version: the version of the package
     """
-    xml_manager = container.xml_manager()
+    from lxml import etree
+    xml_manager = container.xml_manager
     csproj_tree = xml_manager.parse(csproj_file.read_text(encoding="utf-8"))
 
     existing_package_reference = csproj_tree.find(f".//PackageReference[@Include='{name}']")
@@ -83,49 +77,41 @@ def _add_csharp_package_to_csproj(csproj_file: Path, name: str, version: str) ->
     csproj_file.write_text(xml_manager.to_string(csproj_tree), encoding="utf-8")
 
 
-def _add_csharp(project_dir: Path, name: str, version: Optional[str], no_local: bool) -> None:
-    """Adds a custom C# library to a C# project.
+def _add_nuget_package_to_csharp_project(project_dir: Path, name: str, version: Optional[str], no_local: bool) -> None:
+    """Adds a NuGet package to the project in the given directory.
 
-    Adds the library to the project's .csproj file, and restores the project is dotnet is on the user's PATH.
+    Adds the library to the project's .csproj file, and restores the project if dotnet is on the user's PATH.
 
     :param project_dir: the path to the project directory
     :param name: the name of the library to add
     :param version: the version of the library to use, or None to pin to the latest version
     :param no_local: whether restoring the packages locally must be skipped
     """
-    logger = container.logger()
-    path_manager = container.path_manager()
+    logger = container.logger
 
     if version is None:
         logger.info("Retrieving latest available version from NuGet")
         name, version = _get_nuget_package(name)
 
-    csproj_file = next(p for p in project_dir.iterdir() if p.name.endswith(".csproj"))
+    project_manager = container.project_manager
+    csproj_file = project_manager.get_csproj_file_path(project_dir)
+    path_manager = container.path_manager
     logger.info(f"Adding {name} {version} to '{path_manager.get_relative_path(csproj_file)}'")
 
     original_csproj_content = csproj_file.read_text(encoding="utf-8")
     _add_csharp_package_to_csproj(csproj_file, name, version)
-
-    if not no_local and shutil.which("dotnet") is not None:
-        logger.info(
-            f"Restoring packages in '{path_manager.get_relative_path(project_dir)}' to provide local autocomplete")
-
-        process = subprocess.run(["dotnet", "restore", str(csproj_file)], cwd=project_dir)
-
-        if process.returncode != 0:
-            logger.warn(f"Reverting the changes to '{path_manager.get_relative_path(csproj_file)}'")
-            csproj_file.write_text(original_csproj_content, encoding="utf-8")
-
-            raise RuntimeError("Something went wrong while restoring packages, see the logs above for more information")
+    project_manager.try_restore_csharp_project(csproj_file, original_csproj_content, no_local)
 
 
-def _is_pypi_file_compatible(file: Dict[str, Any], required_python_version: StrictVersion) -> bool:
+def _is_pypi_file_compatible(file: Dict[str, Any], required_python_version) -> bool:
     """Checks whether a file on PyPI is compatible with the Python version in the Docker images.
 
     :param file: the data of a file on PyPI, as returned by its JSON API
     :param required_python_version: the Python version to check compatibility for
     :return: True if the file is compatible with the given Python version, False if not
     """
+    from pkg_resources import Requirement
+
     major, minor, patch = required_python_version.version
     if file["python_version"] not in [f"py{major}", f"py{major}{minor}", f"cp{major}", f"cp{major}{minor}", "source"]:
         return False
@@ -147,7 +133,11 @@ def _get_pypi_package(name: str, version: Optional[str]) -> Tuple[str, str]:
     :param version: the version of the package
     :return: a tuple containing the proper name and latest compatible version of the package
     """
-    response = container.http_client().get(f"https://pypi.org/pypi/{name}/json", raise_for_status=False)
+    from json import loads
+    from dateutil.parser import isoparse
+    from distutils.version import StrictVersion
+
+    response = container.http_client.get(f"https://pypi.org/pypi/{name}/json", raise_for_status=False)
 
     if response.status_code == 404:
         raise RuntimeError(f"PyPI does not have a package named {name}")
@@ -155,11 +145,10 @@ def _get_pypi_package(name: str, version: Optional[str]) -> Tuple[str, str]:
     if not response.ok:
         raise RuntimeError(f"The PyPI API is not responding")
 
-    pypi_data = json.loads(response.text)
+    pypi_data = loads(response.text)
     name = pypi_data["info"]["name"]
 
-    required_python_version = "3.6.7" if platform.machine() in ["arm64", "aarch64"] else "3.6.8"
-    required_python_version = StrictVersion(required_python_version)
+    required_python_version = StrictVersion(LEAN_STRICT_PYTHON_VERSION)
 
     last_compatible_version = None
     last_compatible_version_upload_time = None
@@ -198,6 +187,8 @@ def _add_python_package_to_requirements(requirements_file: Path, name: str, vers
     :param name: the name of the package
     :param version: the version of the package
     """
+    from pkg_resources import Requirement
+
     if not requirements_file.is_file():
         requirements_file.touch()
 
@@ -225,7 +216,7 @@ def _add_python_package_to_requirements(requirements_file: Path, name: str, vers
     requirements_file.write_text(new_content, encoding="utf-8")
 
 
-def _add_python(project_dir: Path, name: str, version: Optional[str], no_local: bool) -> None:
+def _add_pypi_package_to_python_project(project_dir: Path, name: str, version: Optional[str], no_local: bool) -> None:
     """Adds a custom Python library to a Python project.
 
     Adds the library to the project's requirements.txt file,
@@ -233,11 +224,13 @@ def _add_python(project_dir: Path, name: str, version: Optional[str], no_local:
 
     :param project_dir: the path to the project directory
     :param name: the name of the library to add
-    :param version: the version of the library to use, or None to pin to the latest version supporting Python 3.6
+    :param version: the version of the library to use, or None to pin to the latest version supporting Python 3.8
     :param no_local: whether installing the package in the local Python environment must be skipped
     """
-    logger = container.logger()
-    path_manager = container.path_manager()
+    from subprocess import run
+    from shutil import which
+    logger = container.logger
+    path_manager = container.path_manager
 
     if version is not None:
         logger.info(f"Checking compatibility of {name} {version} with the Python version used in the Docker images")
@@ -251,31 +244,34 @@ def _add_python(project_dir: Path, name: str, version: Optional[str], no_local:
 
     _add_python_package_to_requirements(requirements_file, name, version)
 
-    if not no_local and shutil.which("pip") is not None:
+    if not no_local and which("pip") is not None:
         logger.info(f"Installing {name} {version} in local Python environment to provide local autocomplete")
 
-        process = subprocess.run(["pip", "install", f"{name}=={version}"])
+        process = run(["pip", "install", f"{name}=={version}"])
 
         if process.returncode != 0:
-            raise RuntimeError(
-                f"Something went wrong while installing {name} {version} locally, see the logs above for more information")
+            raise RuntimeError(f"Something went wrong while installing {name} {version} "
+                               "locally, see the logs above for more information")
 
 
-@click.command(cls=LeanCommand)
-@click.argument("project", type=PathParameter(exists=True, file_okay=False, dir_okay=True))
-@click.argument("name", type=str)
-@click.option("--version", type=str, help="The version of the library to add (defaults to latest compatible version)")
-@click.option("--no-local", is_flag=True, default=False, help="Skip making changes to your local environment")
+@command(cls=LeanCommand)
+@argument("project", type=PathParameter(exists=True, file_okay=False, dir_okay=True))
+@argument("name", type=str)
+@option("--version", type=str, help="The version of the library to add (defaults to latest compatible version)")
+@option("--no-local", is_flag=True, default=False, help="Skip making changes to your local environment")
 def add(project: Path, name: str, version: Optional[str], no_local: bool) -> None:
     """Add a custom library to a project.
 
     PROJECT must be the path to the project.
 
-    NAME must be the name of a NuGet package (for C# projects) or of a PyPI package (for Python projects).
+    NAME must be either the name of a NuGet package (for C# projects), a PyPI package (for Python projects),
+    or a path to a Lean CLI library.
 
-    If --version is not given, the package is pinned to the latest compatible version.
+    If --version is not given, and the library is a NuGet or PyPI package the package, it is pinned to the latest
+    compatible version.
     For C# projects, this is the latest available version.
-    For Python projects, this is the latest version compatible with Python 3.6 (which is what the Docker images use).
+    For Python projects, this is the latest version compatible with Python 3.8 (which is what the Docker images use).
+    For Lean CLI library projects, this is ignored.
 
     Custom C# libraries are added to your project's .csproj file,
     which is then restored if dotnet is on your PATH and the --no-local flag has not been given.
@@ -287,20 +283,34 @@ def add(project: Path, name: str, version: Optional[str], no_local: bool) -> Non
     C# example usage:
     $ lean library add "My CSharp Project" Microsoft.ML
     $ lean library add "My CSharp Project" Microsoft.ML --version 1.5.5
+    $ lean library add "My CSharp Project" "Library/My CSharp Library"
 
     \b
     Python example usage:
     $ lean library add "My Python Project" tensorflow
     $ lean library add "My Python Project" tensorflow --version 2.5.0
+    $ lean library add "My Python Project" "Library/My Python Library"
     """
-    project_config = container.project_config_manager().get_project_config(project)
+    logger = container.logger
+    project_config = container.project_config_manager.get_project_config(project)
     project_language = project_config.get("algorithm-language", None)
 
     if project_language is None:
         raise MoreInfoError(f"{project} is not a Lean CLI project",
-                            "https://www.lean.io/docs/lean-cli/projects/project-management#02-Create-Projects")
+                            "https://www.lean.io/docs/v2/lean-cli/projects/project-management#02-Create-Projects")
 
-    if project_language == "CSharp":
-        _add_csharp(project, name, version, no_local)
+    library_manager = container.library_manager
+    library_dir = Path(name).expanduser().resolve()
+
+    if library_manager.is_lean_library(library_dir):
+        logger.info(f"Adding Lean CLI library {library_dir} to project {project}")
+        if project_language == "CSharp":
+            library_manager.add_lean_library_to_csharp_project(project, library_dir, no_local)
+        else:
+            library_manager.add_lean_library_to_python_project(project, library_dir)
     else:
-        _add_python(project, name, version, no_local)
+        logger.info(f"Adding package {name} to project {project}")
+        if project_language == "CSharp":
+            _add_nuget_package_to_csharp_project(project, name, version, no_local)
+        else:
+            _add_pypi_package_to_python_project(project, name, version, no_local)
