@@ -11,13 +11,13 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import click
+from click import command
 
 from lean.click import LeanCommand
 from lean.container import container
 
 
-@click.command(cls=LeanCommand)
+@command(cls=LeanCommand)
 def logout() -> None:
     """Log out and remove stored credentials."""
-    container.credentials_storage().clear()
+    container.credentials_storage.clear()
