@@ -11,15 +11,11 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import json
-import re
-from datetime import datetime
 from pathlib import Path
 from typing import Optional, List, Tuple
+from datetime import datetime, timedelta
 
-import click
-import json5
-from docker.types import Mount
+from click import command, argument, option, Choice, IntRange
 
 from lean.click import LeanCommand, PathParameter, ensure_options
 from lean.constants import DEFAULT_ENGINE_IMAGE
@@ -29,46 +25,92 @@ from lean.models.errors import MoreInfoError
 from lean.models.optimizer import OptimizationTarget
 
 
-@click.command(cls=LeanCommand, requires_lean_config=True, requires_docker=True)
-@click.argument("project", type=PathParameter(exists=True, file_okay=True, dir_okay=True))
-@click.option("--output",
+def _get_latest_backtest_runtime(algorithm_directory: Path) -> timedelta:
+    from re import findall
+    from dateutil.parser import isoparse
+
+    missing_backtest_error = RuntimeError(
+        "Please run at least one backtest for this project in order to run an optimization estimate");
+    backtests_directory = algorithm_directory / "backtests"
+
+    if not backtests_directory.exists():
+        raise missing_backtest_error
+
+    def is_backtest_output_directory(path: Path) -> bool:
+        try:
+            datetime.strptime(path.name, "%Y-%m-%d_%H-%M-%S")
+        except ValueError:
+            return False
+        else:
+            return path.is_dir()
+
+    def get_filename_timestamp(path: Path) -> datetime:
+        return datetime.strptime(path.name, "%Y-%m-%d_%H-%M-%S")
+
+    backtests_directories = [f for f in backtests_directory.iterdir() if is_backtest_output_directory(f)]
+    latest_backtest_directory = max(backtests_directories, key=get_filename_timestamp, default=None)
+
+    if latest_backtest_directory is None:
+        raise missing_backtest_error
+
+    latest_backtest_log_file = latest_backtest_directory / "log.txt"
+
+    if not latest_backtest_log_file.exists():
+        raise missing_backtest_error
+
+    latest_backtest_logs = latest_backtest_log_file.read_text(encoding="utf-8")
+    timestamps = findall(r"(.+) TRACE:: .*\n", latest_backtest_logs)
+
+    return isoparse(timestamps[-1]) - isoparse(timestamps[0])
+
+
+@command(cls=LeanCommand, requires_lean_config=True, requires_docker=True)
+@argument("project", type=PathParameter(exists=True, file_okay=True, dir_okay=True))
+@option("--output",
               type=PathParameter(exists=False, file_okay=False, dir_okay=True),
               help="Directory to store results in (defaults to PROJECT/optimizations/TIMESTAMP)")
-@click.option("--detach", "-d",
+@option("--detach", "-d",
               is_flag=True,
               default=False,
               help="Run the optimization in a detached Docker container and return immediately")
-@click.option("--optimizer-config",
+@option("--optimizer-config",
               type=PathParameter(exists=True, file_okay=True, dir_okay=False),
               help=f"The optimizer configuration file that should be used")
-@click.option("--strategy",
-              type=click.Choice(["Grid Search", "Euler Search"], case_sensitive=False),
+@option("--strategy",
+              type=Choice(["Grid Search", "Euler Search"], case_sensitive=False),
               help="The optimization strategy to use")
-@click.option("--target",
+@option("--target",
               type=str,
               help="The target statistic of the optimization")
-@click.option("--target-direction",
-              type=click.Choice(["min", "max"], case_sensitive=False),
+@option("--target-direction",
+              type=Choice(["min", "max"], case_sensitive=False),
               help="Whether the target must be minimized or maximized")
-@click.option("--parameter",
+@option("--parameter",
               type=(str, float, float, float),
               multiple=True,
               help="The 'parameter min max step' pairs configuring the parameters to optimize")
-@click.option("--constraint",
+@option("--constraint",
               type=str,
               multiple=True,
               help="The 'statistic operator value' pairs configuring the constraints of the optimization")
-@click.option("--release",
+@option("--release",
               is_flag=True,
               default=False,
               help="Compile C# projects in release configuration instead of debug")
-@click.option("--image",
+@option("--image",
               type=str,
               help=f"The LEAN engine image to use (defaults to {DEFAULT_ENGINE_IMAGE})")
-@click.option("--update",
+@option("--update",
               is_flag=True,
               default=False,
               help="Pull the LEAN engine image before running the optimizer")
+@option("--estimate",
+              is_flag=True,
+              default=False,
+              help="Estimate optimization runtime without running it")
+@option("--max-concurrent-backtests",
+              type=IntRange(min=1),
+              help="Maximum number of concurrent backtests to run")
 def optimize(project: Path,
              output: Optional[Path],
              detach: bool,
@@ -80,7 +122,9 @@ def optimize(project: Path,
              constraint: List[str],
              release: bool,
              image: Optional[str],
-             update: bool) -> None:
+             update: bool,
+             estimate: bool,
+             max_concurrent_backtests: Optional[int]) -> None:
     """Optimize a project's parameters locally using Docker.
 
     \b
@@ -109,24 +153,41 @@ def optimize(project: Path,
     - --constraint "<statistic> <operator> <value>"
     - --constraint "Sharpe Ratio >= 0.5" --constraint "Drawdown < 0.25"
 
+    \b
+    If --estimate is given, the optimization will not be executed.
+    The runtime estimate for the optimization will be calculated and outputted.
+
     By default the official LEAN engine image is used.
     You can override this using the --image option.
     Alternatively you can set the default engine image for all commands using `lean config set engine-image <image>`.
     """
-    project_manager = container.project_manager()
+    from json import dumps
+    from json5 import loads
+    from docker.types import Mount
+    from re import findall, search
+    from os import cpu_count
+    from math import floor
+
+    should_detach = detach and not estimate
+
+    project_manager = container.project_manager
     algorithm_file = project_manager.find_algorithm_file(project)
 
+    latest_backtest_runtime = timedelta(0)
+    if estimate:
+        latest_backtest_runtime = _get_latest_backtest_runtime(algorithm_file.parent)
+
     if output is None:
         output = algorithm_file.parent / "optimizations" / datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
 
-    optimizer_config_manager = container.optimizer_config_manager()
+    optimizer_config_manager = container.optimizer_config_manager
     config = None
 
     if optimizer_config is not None and strategy is not None:
         raise RuntimeError("--optimizer-config and --strategy are mutually exclusive")
 
     if optimizer_config is not None:
-        config = json5.loads(optimizer_config.read_text(encoding="utf-8"))
+        config = loads(optimizer_config.read_text(encoding="utf-8"))
 
         # Remove keys which are configured in the Lean config
         for key in ["algorithm-type-name", "algorithm-language", "algorithm-location"]:
@@ -140,13 +201,13 @@ def optimize(project: Path,
         optimization_parameters = optimizer_config_manager.parse_parameters(parameter)
         optimization_constraints = optimizer_config_manager.parse_constraints(constraint)
     else:
-        project_config_manager = container.project_config_manager()
+        project_config_manager = container.project_config_manager
         project_config = project_config_manager.get_project_config(algorithm_file.parent)
         project_parameters = [QCParameter(key=k, value=v) for k, v in project_config.get("parameters", {}).items()]
 
         if len(project_parameters) == 0:
             raise MoreInfoError("The given project has no parameters to optimize",
-                                "https://www.lean.io/docs/lean-cli/optimization/parameters")
+                                "https://www.lean.io/docs/v2/lean-cli/optimization/parameters")
 
         optimization_strategy = optimizer_config_manager.configure_strategy(cloud=False)
         optimization_target = optimizer_config_manager.configure_target()
@@ -169,55 +230,65 @@ def optimize(project: Path,
             "constraints": [constraint.dict(by_alias=True) for constraint in optimization_constraints]
         }
 
+    if max_concurrent_backtests is not None:
+        config["maximum-concurrent-backtests"] = max_concurrent_backtests
+    elif "maximum-concurrent-backtests" in config:
+        max_concurrent_backtests = config["maximum-concurrent-backtests"]
+    else:
+        max_concurrent_backtests = max(1, floor(cpu_count() / 2))
+
     config["optimizer-close-automatically"] = True
     config["results-destination-folder"] = "/Results"
 
     config_path = output / "optimizer-config.json"
     config_path.parent.mkdir(parents=True, exist_ok=True)
     with config_path.open("w+", encoding="utf-8") as file:
-        file.write(json.dumps(config, indent=4) + "\n")
+        file.write(dumps(config, indent=4) + "\n")
 
-    project_config_manager = container.project_config_manager()
-    cli_config_manager = container.cli_config_manager()
+    project_config_manager = container.project_config_manager
+    cli_config_manager = container.cli_config_manager
 
     project_config = project_config_manager.get_project_config(algorithm_file.parent)
     engine_image = cli_config_manager.get_engine_image(image or project_config.get("engine-image", None))
 
-    lean_config_manager = container.lean_config_manager()
+    logger = container.logger
+
+    if str(engine_image) != DEFAULT_ENGINE_IMAGE:
+        logger.warn(f'A custom engine image: "{engine_image}" is being used!')
+
+    lean_config_manager = container.lean_config_manager
     lean_config = lean_config_manager.get_complete_lean_config("backtesting", algorithm_file, None)
 
     if not output.exists():
         output.mkdir(parents=True)
 
-    output_config_manager = container.output_config_manager()
+    output_config_manager = container.output_config_manager
     lean_config["algorithm-id"] = str(output_config_manager.get_optimization_id(output))
     lean_config["messaging-handler"] = "QuantConnect.Messaging.Messaging"
 
-    lean_runner = container.lean_runner()
-    run_options = lean_runner.get_basic_docker_config(lean_config, algorithm_file, output, None, release, detach)
+    lean_runner = container.lean_runner
+    run_options = lean_runner.get_basic_docker_config(lean_config, algorithm_file, output, None, release, should_detach)
 
     run_options["working_dir"] = "/Lean/Optimizer.Launcher/bin/Debug"
-    run_options["commands"].append("dotnet QuantConnect.Optimizer.Launcher.dll")
+    run_options["commands"].append(f"dotnet QuantConnect.Optimizer.Launcher.dll{' --estimate' if estimate else ''}")
     run_options["mounts"].append(
         Mount(target="/Lean/Optimizer.Launcher/bin/Debug/config.json",
               source=str(config_path),
               type="bind",
               read_only=True)
     )
-
-    container.update_manager().pull_docker_image_if_necessary(engine_image, update)
+    container.update_manager.pull_docker_image_if_necessary(engine_image, update)
 
     project_manager.copy_code(algorithm_file.parent, output / "code")
 
-    success = container.docker_manager().run_image(engine_image, **run_options)
+    success = container.docker_manager.run_image(engine_image, **run_options)
 
-    logger = container.logger()
-    cli_root_dir = container.lean_config_manager().get_cli_root_directory()
+    cli_root_dir = container.lean_config_manager.get_cli_root_directory()
     relative_project_dir = project.relative_to(cli_root_dir)
     relative_output_dir = output.relative_to(cli_root_dir)
 
-    if detach:
-        temp_manager = container.temp_manager()
+    if should_detach:
+        temp_manager = container.temp_manager
         temp_manager.delete_temporary_directories_when_done = False
 
         logger.info(
@@ -226,27 +297,41 @@ def optimize(project: Path,
         logger.info("You can use Docker's own commands to manage the detached container")
     elif success:
         optimizer_logs = (output / "log.txt").read_text(encoding="utf-8")
-        groups = re.findall(r"ParameterSet: \(([^)]+)\) backtestId '([^']+)'", optimizer_logs)
 
-        if len(groups) > 0:
-            optimal_parameters, optimal_id = groups[0]
+        if estimate:
+            match = search(r"Optimization estimate: (\d+)", optimizer_logs)
 
-            optimal_results = json.loads((output / optimal_id / f"{optimal_id}.json").read_text(encoding="utf-8"))
-            optimal_backtest = QCBacktest(backtestId=optimal_id,
-                                          projectId=1,
-                                          status="",
-                                          name=optimal_id,
-                                          created=datetime.now(),
-                                          completed=True,
-                                          progress=1.0,
-                                          runtimeStatistics=optimal_results["RuntimeStatistics"],
-                                          statistics=optimal_results["Statistics"])
+            if match is None:
+                raise RuntimeError(f"Something went wrong while running the optimization estimate, "
+                                   f"the output is stored in '{relative_output_dir}'")
 
-            logger.info(f"Optimal parameters: {optimal_parameters.replace(':', ': ').replace(',', ', ')}")
-            logger.info(f"Optimal backtest results:")
-            logger.info(optimal_backtest.get_statistics_table())
+            backtestsCount = int(match[1])
+            logger.info(f"Optimization estimate: \n"
+                        f"  Total backtests: {backtestsCount}\n"
+                        f"  Estimated runtime: {backtestsCount * latest_backtest_runtime / max_concurrent_backtests}")
+        else:
+            groups = findall(r"ParameterSet: \(([^)]+)\) backtestId '([^']+)'", optimizer_logs)
 
-        logger.info(f"Successfully optimized '{relative_project_dir}' and stored the output in '{relative_output_dir}'")
+            if len(groups) > 0:
+                optimal_parameters, optimal_id = groups[0]
+
+                optimal_results = loads((output / optimal_id / f"{optimal_id}.json").read_text(encoding="utf-8"))
+                optimal_backtest = QCBacktest(backtestId=optimal_id,
+                                              projectId=1,
+                                              status="",
+                                              name=optimal_id,
+                                              created=datetime.now(),
+                                              completed=True,
+                                              progress=1.0,
+                                              runtimeStatistics=optimal_results["RuntimeStatistics"],
+                                              statistics=optimal_results["Statistics"])
+
+                logger.info(f"Optimal parameters: {optimal_parameters.replace(':', ': ').replace(',', ', ')}")
+                logger.info(f"Optimal backtest results:")
+                logger.info(optimal_backtest.get_statistics_table())
+
+            logger.info(
+                f"Successfully optimized '{relative_project_dir}' and stored the output in '{relative_output_dir}'")
     else:
         raise RuntimeError(
             f"Something went wrong while running the optimization, the output is stored in '{relative_output_dir}'")
