@@ -12,11 +12,10 @@
 # limitations under the License.
 
 from datetime import datetime
-from math import floor
 from typing import List, Optional
 
 from lean.components.api.api_client import *
-from lean.models.api import QCFullLiveAlgorithm, QCLiveAlgorithmStatus, QCMinimalLiveAlgorithm, QCNotificationMethod
+from lean.models.api import QCFullLiveAlgorithm, QCLiveAlgorithmStatus, QCMinimalLiveAlgorithm, QCNotificationMethod, QCRestResponse
 
 
 class LiveClient:
@@ -41,6 +40,7 @@ class LiveClient:
         :param end: the latest launch time the returned algorithms should have
         :return: a list of live algorithms which match the given filters
         """
+        from math import floor
         parameters = {
             "start": floor(start.timestamp()),
             "end": floor(end.timestamp())
@@ -56,13 +56,15 @@ class LiveClient:
               project_id: int,
               compile_id: str,
               node_id: str,
-              brokerage_settings: Dict[str, str],
+              brokerage_settings: Dict[str, Any],
               price_data_handler: str,
               automatic_redeploy: bool,
               version_id: int,
               notify_order_events: bool,
               notify_insights: bool,
-              notify_methods: List[QCNotificationMethod]) -> QCMinimalLiveAlgorithm:
+              notify_methods: List[QCNotificationMethod],
+              live_cash_balance: Optional[List[Dict[str, float]]] = None,
+              live_holdings: Optional[List[Dict[str, float]]] = None) -> QCMinimalLiveAlgorithm:
         """Starts live trading for a project.
 
         :param project_id: the id of the project to start live trading for
@@ -75,9 +77,16 @@ class LiveClient:
         :param notify_order_events: whether notifications should be sent on order events
         :param notify_insights: whether notifications should be sent on insights
         :param notify_methods: the places to send notifications to
+        :param live_cash_balance: the list of initial cash balance
+        :param live_holdings: the list of initial portfolio holdings
         :return: the created live algorithm
         """
 
+        if live_cash_balance:
+            brokerage_settings["cash"] = live_cash_balance
+        if live_holdings:
+            brokerage_settings["holdings"] = live_holdings
+
         parameters = {
             "projectId": project_id,
             "compileId": compile_id,
@@ -97,26 +106,28 @@ class LiveClient:
 
             parameters["notification"] = {
                 "events": events,
-                "targets": [method.dict() for method in notify_methods]
+                "targets": [{x: y for x, y in method.dict().items() if y} for method in notify_methods]
             }
 
         data = self._api.post("live/create", parameters)
         return QCMinimalLiveAlgorithm(**data)
 
-    def stop(self, project_id: int) -> None:
+    def stop(self, project_id: int) -> QCRestResponse:
         """Stops live trading for a certain project without liquidated existing positions.
 
         :param project_id: the id of the project to stop live trading for
         """
-        self._api.post("live/update/stop", {
+        data = self._api.post("live/update/stop", {
             "projectId": project_id
         })
+        return QCRestResponse(**data)
 
-    def liquidate_and_stop(self, project_id: int) -> None:
+    def liquidate_and_stop(self, project_id: int) -> QCRestResponse:
         """Stops live trading and liquidates existing positions for a certain project.
 
         :param project_id: the id of the project to stop live trading and liquidate existing positions for
         """
-        self._api.post("live/update/liquidate", {
+        data = self._api.post("live/update/liquidate", {
             "projectId": project_id
         })
+        return QCRestResponse(**data)
