@@ -11,11 +11,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import re
 from pathlib import Path
 from typing import Any, Dict, Optional, List
 
-import json5
 
 from lean.components.cloud.module_manager import ModuleManager
 from lean.components.config.cli_config_manager import CLIConfigManager
@@ -74,13 +72,14 @@ class LeanConfigManager:
             target_file = current_dir / DEFAULT_LEAN_CONFIG_FILE_NAME
             if target_file.exists():
                 self._lean_config_path = target_file
+
                 self.store_known_lean_config_path(self._lean_config_path)
                 return self._lean_config_path
 
             # If the parent directory is the same as the current directory we can't go up any more
             if current_dir.parent == current_dir:
                 raise MoreInfoError(f"'{DEFAULT_LEAN_CONFIG_FILE_NAME}' not found",
-                                    "https://www.lean.io/docs/lean-cli/initialization/configuration#03-Lean-Configuration")
+                                    "https://www.lean.io/docs/v2/lean-cli/initialization/configuration#03-Lean-Configuration")
 
             current_dir = current_dir.parent
 
@@ -137,17 +136,20 @@ class LeanConfigManager:
 
         :param updates: the key -> new value updates to apply to the current config
         """
+        from json5 import dumps
+        from re import sub
+
         config = self.get_lean_config()
 
         config_path = self.get_lean_config_path()
         config_text = config_path.read_text(encoding="utf-8")
 
         for key, value in reversed(list(updates.items())):
-            json_value = json5.dumps(value)
+            json_value = dumps(value)
 
             # We can only use regex to set the property because converting the config back to JSON drops all comments
             if key in config:
-                config_text = re.sub(fr'"{key}":\s*("?[^",]*"?)', f'"{key}": {json_value}', config_text)
+                config_text = sub(fr'"{key}":\s*("?[^",]*"?)', f'"{key}": {json_value}', config_text)
             else:
                 config_text = config_text.replace("{", f'{{\n  "{key}": {json_value},', 1)
 
@@ -178,6 +180,8 @@ class LeanConfigManager:
         :param config: the configuration to remove the auto-configurable keys from
         :return: the same config as passed in with the config argument, but without the auto-configurable keys
         """
+        from re import split
+
         # The keys that we can set automatically based on the command that is ran
         keys_to_remove = ["environment",
                           "composer-dll-directory",
@@ -188,7 +192,7 @@ class LeanConfigManager:
 
         # This function is implemented by doing string manipulation because the config contains comments
         # If we were to parse it as JSON, we would have to remove the comments, which we don't want to do
-        sections = re.split(r"\n\s*\n", config)
+        sections = split(r"\n\s*\n", config)
         for key in keys_to_remove:
             sections = [section for section in sections if f"\"{key}\": " not in section]
         config = "\n\n".join(sections)
@@ -254,14 +258,25 @@ class LeanConfigManager:
             config["algorithm-language"] = "Python"
             config["algorithm-location"] = f"/LeanCLI/{algorithm_file.name}"
         else:
+            from re import findall
             algorithm_text = algorithm_file.read_text(encoding="utf-8")
-            config["algorithm-type-name"] = re.findall(r"class\s*([^\s:]+)\s*:\s*QCAlgorithm", algorithm_text)[0]
+            config["algorithm-type-name"] = findall(r"class\s*([^\s:]+)\s*:\s*QCAlgorithm", algorithm_text)[0]
             config["algorithm-language"] = "CSharp"
             config["algorithm-location"] = f"{algorithm_file.parent.name}.dll"
 
         project_config = self._project_config_manager.get_project_config(algorithm_file.parent)
         config["parameters"] = project_config.get("parameters", {})
 
+        # Add libraries paths to python project
+        project_language = project_config.get("algorithm-language", None)
+        if project_language == "Python":
+            library_references = project_config.get("libraries", [])
+            python_paths = config.get("python-additional-paths", [])
+            python_paths.extend([(Path("/") / library["path"]).as_posix() for library in library_references])
+            if len(python_paths) > 0:
+                python_paths.append("/Library")
+            config["python-additional-paths"] = python_paths
+
         # No real limit for the object store by default
         if "storage-limit-mb" not in config:
             config["storage-limit-mb"] = "9999999"
@@ -293,4 +308,39 @@ class LeanConfigManager:
 
         :return: a dict containing the contents of the Lean config file
         """
-        return json5.loads(self.get_lean_config_path().read_text(encoding="utf-8"))
+        path = self.get_lean_config_path()
+        content = path.read_text(encoding="utf-8")
+        return self.parse_json(content)
+
+    def parse_json(self, content) -> Dict[str, Any]:
+        try:
+            from json import loads
+            from re import sub
+
+            # remove multi line or single line comments without double quotes
+            config = sub(r'/\*.*?\*/|//[^\r\n"]*[\r\n]', '', content)
+
+            # let's handle single line comments with double quotes in them
+            new_config = ''
+            for line in config.split('\n'):
+                double_quotes_count = 0
+                previous_element = ''
+                for current_element in line:
+                    if current_element == '/' and double_quotes_count % 2 == 0:
+                        if previous_element == '/':
+                            break
+                    else:
+                        # count not escaped double quotes
+                        if current_element == '"' and previous_element != '\\':
+                            double_quotes_count = double_quotes_count + 1
+                        new_config += current_element
+                    previous_element = current_element
+            result = loads(new_config)
+            return result
+
+        except Exception as e:
+            self._logger.error(str(e))
+
+            # just in case slower fallback
+            from json5 import loads
+            return loads(content)
