@@ -11,9 +11,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import json
-
-import requests
 
 from lean.components.util.logger import Logger
 
@@ -28,7 +25,7 @@ class HTTPClient:
         """
         self._logger = logger
 
-    def get(self, url: str, **kwargs) -> requests.Response:
+    def get(self, url: str, **kwargs):
         """A wrapper around requests.get().
 
         An error is raised if the response is unsuccessful unless kwargs["raise_for_status"] == False.
@@ -37,15 +34,9 @@ class HTTPClient:
         :param kwargs: any kwargs to pass on to requests.get()
         :return: the response of the request
         """
-        self._log_request("GET", url, **kwargs)
+        return self.request("GET", url, **kwargs)
 
-        raise_for_status = kwargs.pop("raise_for_status", True)
-        response = requests.get(url, **kwargs)
-
-        self._check_response(response, raise_for_status)
-        return response
-
-    def post(self, url: str, **kwargs) -> requests.Response:
+    def post(self, url: str, **kwargs):
         """A wrapper around requests.post().
 
         An error is raised if the response is unsuccessful unless kwargs["raise_for_status"] == False.
@@ -54,15 +45,9 @@ class HTTPClient:
         :param kwargs: any kwargs to pass on to requests.post()
         :return: the response of the request
         """
-        self._log_request("POST", url, **kwargs)
+        return self.request("POST", url, **kwargs)
 
-        raise_for_status = kwargs.pop("raise_for_status", True)
-        response = requests.post(url, **kwargs)
-
-        self._check_response(response, raise_for_status)
-        return response
-
-    def request(self, method: str, url: str, **kwargs) -> requests.Response:
+    def request(self, method: str, url: str, **kwargs):
         """A wrapper around requests.request().
 
         An error is raised if the response is unsuccessful unless kwargs["raise_for_status"] == False.
@@ -72,15 +57,25 @@ class HTTPClient:
         :param kwargs: any kwargs to pass on to requests.request()
         :return: the response of the request
         """
+        from requests import request, exceptions
+
         self._log_request(method, url, **kwargs)
 
         raise_for_status = kwargs.pop("raise_for_status", True)
-        response = requests.request(method, url, **kwargs)
+        try:
+            response = request(method, url, **kwargs)
+        except exceptions.SSLError as e:
+            raise Exception(f"""
+Detected SSL error, this might be due to custom certificates in your environment or system trust store.
+A known limitation of the python requests implementation.
+Please consider installing library https://pypi.org/project/python-certifi-win32/.
+Related issue https://github.com/psf/requests/issues/2966
+    """.strip())
 
         self._check_response(response, raise_for_status)
         return response
 
-    def log_unsuccessful_response(self, response: requests.Response) -> None:
+    def log_unsuccessful_response(self, response) -> None:
         """Logs an unsuccessful response's status code and body.
 
         :param response: the response to log
@@ -95,15 +90,16 @@ class HTTPClient:
         :param url: the request url
         :param kwargs: any kwargs passed to a request.* method
         """
+        from json import dumps
         message = f"--> {method.upper()} {url}"
 
         data = next((kwargs.get(key) for key in ["json", "data", "params"] if key in kwargs), None)
         if data is not None and data != {}:
-            message += f" with data:\n{json.dumps(data, indent=4)}"
+            message += f" with data:\n{dumps(data, indent=4)}"
 
         self._logger.debug(message)
 
-    def _check_response(self, response: requests.Response, raise_for_status: bool) -> None:
+    def _check_response(self, response, raise_for_status: bool) -> None:
         """Checks a response, logging a debug message if it wasn't successful.
 
         :param response: the response to check
