@@ -12,15 +12,14 @@
 # limitations under the License.
 
 from pathlib import Path
-import re
 from typing import Any, Dict, List
-import click
-import abc
+from click import prompt, Choice
+from abc import ABC, abstractmethod
 from lean.components.util.logger import Logger
 from lean.click import PathParameter
 
 
-class BaseCondition(abc.ABC):
+class BaseCondition(ABC):
     """Base condition class extended to all types of conditions"""
 
     def __init__(self, condition_object: Dict[str, str]):
@@ -43,7 +42,7 @@ class BaseCondition(abc.ABC):
             raise ValueError(
                 f'Undefined condition type {condition_object["type"]}')
 
-    @abc.abstractmethod
+    @abstractmethod
     def check(self, target_value: str) -> bool:
         """validates the condition against the provided values
 
@@ -74,7 +73,8 @@ class RegexCondition(BaseCondition):
         :param target_value: value to validate the condition against
         :return: True if the condition is valid otherwise False
         """
-        return len(re.findall(self._pattern, target_value, re.I)) > 0
+        from re import findall, I
+        return len(findall(self._pattern, target_value, I)) > 0
 
 
 class ConditionalValueOption():
@@ -86,19 +86,18 @@ class ConditionalValueOption():
             option_object["condition"])
 
 
-class Configuration(abc.ABC):
+class Configuration(ABC):
     """Base configuration class extended to all types of configurations"""
 
     def __init__(self, config_json_object):
         self._id: str = config_json_object["id"]
         self._config_type: str = config_json_object["type"]
         self._value: str = config_json_object["value"]
+        self._is_cloud_property: bool = "cloud-id" in config_json_object
         self._is_required_from_user = False
         self._is_type_configurations_env: bool = type(
             self) is ConfigurationsEnvConfiguration
         self._is_type_trading_env: bool = type(self) is TradingEnvConfiguration
-        self.is_type_organization_id: bool = type(
-            self) is OrganzationIdConfiguration
         self._log_message: str = ""
         if "log-message" in config_json_object.keys():
             self._log_message = config_json_object["log-message"]
@@ -119,10 +118,10 @@ class Configuration(abc.ABC):
             return InfoConfiguration.factory(config_json_object)
         elif config_json_object["type"] in ["input", "internal-input"]:
             return UserInputConfiguration.factory(config_json_object)
-        elif config_json_object["type"] in ["filter-env", "trading-env"]:
+        elif config_json_object["type"] == "filter-env":
             return BrokerageEnvConfiguration.factory(config_json_object)
-        elif config_json_object["type"] == "organization-id":
-            return OrganzationIdConfiguration(config_json_object)
+        elif config_json_object["type"] == "trading-env":
+            return TradingEnvConfiguration.factory(config_json_object)
         else:
             raise ValueError(
                 f'Undefined input method type {config_json_object["type"]}')
@@ -173,7 +172,7 @@ class ConfigurationsEnvConfiguration(InfoConfiguration):
             env_obj["name"]: env_obj["value"] for env_obj in self._value}
 
 
-class UserInputConfiguration(Configuration, abc.ABC):
+class UserInputConfiguration(Configuration, ABC):
     """Base class extended to all confiugration class than store values in Lean config.
 
     Values are expected from the user via prompts.
@@ -197,7 +196,7 @@ class UserInputConfiguration(Configuration, abc.ABC):
         if "cloud-id" in config_json_object.keys():
             self._cloud_id = config_json_object["cloud-id"]
 
-    @abc.abstractmethod
+    @abstractmethod
     def AskUserForInput(self, default_value: Any, logger: Logger):
         """Prompts user to provide input while validating the type of input
         against the expected type
@@ -276,7 +275,7 @@ class PromptUserInput(UserInputConfiguration):
         :param logger: The instance of logger class.
         :return: The value provided by the user.
         """
-        return click.prompt(self._prompt_info, default_value, type=self.get_input_type())
+        return prompt(self._prompt_info, default_value, type=self.get_input_type())
 
     def get_input_type(self):
         return self.map_to_types.get(self._input_type, self._input_type)
@@ -297,10 +296,10 @@ class ChoiceUserInput(UserInputConfiguration):
         :param logger: The instance of logger class.
         :return: The value provided by the user.
         """
-        return click.prompt(
+        return prompt(
             self._prompt_info,
             default_value,
-            type=click.Choice(self._choices, case_sensitive=False)
+            type=Choice(self._choices, case_sensitive=False)
         )
 
 
@@ -324,7 +323,7 @@ class PathParameterUserInput(UserInputConfiguration):
             default_binary = Path(self._input_default)
         else:
             default_binary = ""
-        value = click.prompt(self._prompt_info,
+        value = prompt(self._prompt_info,
                              default=default_binary,
                              type=PathParameter(
                                  exists=False, file_okay=True, dir_okay=False)
@@ -344,7 +343,7 @@ class ConfirmUserInput(UserInputConfiguration):
         :param logger: The instance of logger class.
         :return: The value provided by the user.
         """
-        return click.prompt(self._prompt_info, default_value, type=bool)
+        return prompt(self._prompt_info, default_value, type=bool)
 
 
 class PromptPasswordUserInput(UserInputConfiguration):
@@ -362,13 +361,6 @@ class PromptPasswordUserInput(UserInputConfiguration):
         return logger.prompt_password(self._prompt_info, default_value)
 
 
-class OrganzationIdConfiguration(PromptUserInput):
-    """This class is used for job-organzation-id configurations"""
-
-    def __init__(self, config_json_object):
-        super().__init__(config_json_object)
-
-
 class BrokerageEnvConfiguration(PromptUserInput, ChoiceUserInput, ConfirmUserInput):
     """This class is base class extended by all classes that needs to add value to user filters"""
 
@@ -379,11 +371,9 @@ class BrokerageEnvConfiguration(PromptUserInput, ChoiceUserInput, ConfirmUserInp
         """Creates an instance of the child classes.
 
         :param config_json_object: the json object dict with configuration info
-        :return: An instance of BrokerageEnvConfiguration.
+        :return: An instance of BrokerageEnvConfiguration
         """
-        if config_json_object["type"] == "trading-env":
-            return TradingEnvConfiguration(config_json_object)
-        elif config_json_object["type"] == "filter-env":
+        if config_json_object["type"] == "filter-env":
             return FilterEnvConfiguration(config_json_object)
         else:
             raise ValueError(
@@ -408,15 +398,27 @@ class BrokerageEnvConfiguration(PromptUserInput, ChoiceUserInput, ConfirmUserInp
                 f"Undefined input method type {self._input_method}")
 
 
-class TradingEnvConfiguration(BrokerageEnvConfiguration):
+class TradingEnvConfiguration(PromptUserInput, ChoiceUserInput, ConfirmUserInput):
     """This class adds trading-mode/envirionment based user filters.
-    
-    Normalizes the value of envrionment values(live/paper) for cloud live. 
+
+    Normalizes the value of envrionment values(live/paper) for cloud live.
     """
 
     def __init__(self, config_json_object):
         super().__init__(config_json_object)
 
+    def factory(config_json_object) -> 'TradingEnvConfiguration':
+        """Creates an instance of the child classes.
+
+        :param config_json_object: the json object dict with configuration info
+        :return: An instance of TradingEnvConfiguration.
+        """
+        if config_json_object["type"] == "trading-env":
+            return TradingEnvConfiguration(config_json_object)
+        else:
+            raise ValueError(
+                f'Undefined input method type {config_json_object["type"]}')
+
     def AskUserForInput(self, default_value, logger: Logger):
         """Prompts user to provide input while validating the type of input
         against the expected type
